/*  jview : Image viewer
 *  Copyright (C) 1998 david Ramboz
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __BROWSERS_H__
#define __BROWSERS_H__

#include <sys/time.h>
#include <unistd.h>
#include <gtk/gtk.h>

#include "image.h"

/* 
   Image Browser 
*/

#define FIRST_PAGE    -1
#define LAST_PAGE     -2
#define NEXT_PAGE     -3
#define PREVIOUS_PAGE -4

#define NEXT_IMAGE     -1
#define PREVIOUS_IMAGE -2

typedef struct _ImageBrowser ImageBrowser;
typedef void (*ImageBrowserWaitCallback) (  ImageBrowser *browser);

struct _ImageBrowser {
    GtkWidget *mainbox;
    GtkWidget *preview_box;
    GtkWidget *viewer_box;
    GtkWidget *progress_bar;
  
    GtkWidget *image_viewer;
    GtkWidget *lbl_image_name;
    GtkWidget *lbl_image_dim;
    GtkWidget *lbl_image_size;
    GtkWidget *lbl_page_info;

    GList *images;
    gint  image_count;
    gint  images_per_page;
    GFunc image_free;

    gint image_no;
    gint page_no;

    gint preview_width;
    gint preview_height;

    GCompareFunc compare;
    gint         reverse_sort : 1;

    guint  diaporama_timeout;
    struct timeval diaporama_start;
    gint   diaporama_tag;

    gint making_previews : 1;
    gint stop_making_previews : 1;

    ImageBrowserWaitCallback wait_callback;
    gpointer callback_data;

    gint ww, wh, wx, wy; /* saved window geometry */
};

ImageBrowser*           image_browser_new( GFunc image_free_func);
void                    image_browser_set_images( ImageBrowser *browser, 
						  GList *images);
void                    image_browser_append_image( ImageBrowser *browser, 
						    Image *image);
void                    image_browser_goto_page( ImageBrowser *browser, 
						 gint page);
void                    image_browser_view_image( ImageBrowser *browser,
						  gint image);
GtkItemFactory*         image_browser_get_menu_factory( ImageBrowser *browser);
/* 
   Directory Browser
*/

typedef struct _DirectoryBrowser DirectoryBrowser;

struct _DirectoryBrowser {
    GtkWidget    *file_browser;
    ImageBrowser *image_browser;

    GList *images;
    char *path;
};

GtkWidget*              directory_browser_new( char *starting_directory,
					       gint preview_width,
					       gint preview_height,
					       gint images_per_page);


void                    main_quit();
    
#endif /* __BROWSERS_H__ */
