/*  jview : Image viewer
 *  Copyright (C) 1998 david Ramboz
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __FILE_BROWSER_H__
#define __FILE_BROWSER_H__


#include <gtk/gtkctree.h>


#define TYPE_FILE_BROWSER            (file_browser_get_type())
#define FILE_BROWSER(obj)            (GTK_CHECK_CAST ((obj), TYPE_FILE_BROWSER, FileBrowser))
#define FILE_BROWSER_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), TYPE_FILE_BROWSER, FileBrowserClass))
#define IS_FILE_BROWSER(obj)         (GTK_CHECK_TYPE ((obj), TYPE_FILE_BROWSER))
#define IS_FILE_BROWSER_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), TYPE_FILE_BROWSER))


typedef struct _FileBrowser FileBrowser;
typedef struct _FileBrowserClass FileBrowserClass;

typedef enum {
    FILE_BROWSER_SHOW_FILES     = 1 << 0, /* if FALSE show only directories */
    FILE_BROWSER_LONG_LIST      = 1 << 1, /* long listing like ls -l */
    FILE_BROWSER_SHOW_ALL       = 1 << 2, /* list all files, even those begining with '.' */
    FILE_BROWSER_SHOW_CTIME     = 1 << 3, /*  |                                      |  */
    FILE_BROWSER_SHOW_MTIME     = 1 << 4, /* <  Only 1 of these flags should be set   > */
    FILE_BROWSER_SHOW_ATIME     = 1 << 5, /*  |                                      |  */
    FILE_BROWSER_IGNORE_BACKUPS = 1 << 6  /* ignore files that ends with '~' */
} FileBrowserFlags;

struct _FileBrowser {
    GtkCTree tree;

    guint flags;

    char last_path[4096];
};

struct _FileBrowserClass {
    GtkCTreeClass parent_class;
    
    void (*select_file) ( FileBrowser *file_browser,
			  char *file_path);
};


GtkType             file_browser_get_type( void);
GtkWidget*          file_browser_new( char *root_path,
					  guint flags);


#endif /* __FILE_BROWSER_H__ */
