/*  jview : Image viewer
 *  Copyright (C) 1998 david Ramboz
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <config.h>
#ifdef HAVE_LIBGIF

#include <stdio.h>
#include <unistd.h>
#include <gif_lib.h>
#include "gif.h"


void
register_gif() {
    char *extensions[] = { "gif", NULL};
    ImageFormat gif_format = {
	extensions,
	is_gif,
	gif_read_header,
	gif_read,
	NULL
    };

    image_format_register( &gif_format);
}

gint
is_gif( Image *image) {
    char buff[4];

    if ( !fread( buff, 3, 1, image->preview->file))
	return FALSE;

    fseek( image->preview->file, 0, SEEK_SET);

    buff[3] = 0;
    return !strcmp( buff, "GIF");
}

gint
gif_read_header( Image *image) {
    GifFileType *gif;
    GifRecordType rec;
    GifByteType *ext;
    gint err = FALSE, ext_code, has_transp = FALSE;
    gint width = 0, height;

    lseek( fileno( image->preview->file), 0, SEEK_SET);
    gif = DGifOpenFileHandle( fileno( image->preview->file));

    if ( !gif)
	return FALSE;

    /* find if image has transparency */
    do {
	if ( DGifGetRecordType( gif, &rec) == GIF_ERROR) {
	    err = TRUE;
	    break;
	}
	switch( rec) {
	case SCREEN_DESC_RECORD_TYPE:
	    /* skip */
	    break;
	case IMAGE_DESC_RECORD_TYPE:
	    if ( DGifGetImageDesc(gif) == GIF_ERROR) {
		err = TRUE;
		break;
	    }
		
	    width = gif->Image.Width;
	    height = gif->Image.Height;

	    rec = TERMINATE_RECORD_TYPE;
	    break;
	case EXTENSION_RECORD_TYPE:
	    if ( DGifGetExtension(gif, &ext_code, &ext) == GIF_ERROR) {
		err = TRUE;
		break;
	    }
	    do {
		if ( ext && (ext[1] & 1)) {
		    has_transp = TRUE;
		    if ( width > 0)
			rec = TERMINATE_RECORD_TYPE;
		    break;
		}
	    } while ( DGifGetExtensionNext( gif, &ext) == GIF_OK && ext);
	    break;
	case TERMINATE_RECORD_TYPE:
	default:
	    break;
	}

    } while ( !err && rec != TERMINATE_RECORD_TYPE);

    DGifCloseFile( gif);

    if ( !err)
	image_set_infos( image,
			 GIF_IMAGE,
			 width, height,
			 TRUE,
			 has_transp);

    return err == FALSE;
}

gint
gif_read( Image *image) {
    GifFileType *gif;
    GifRecordType rec;
    GifByteType *ext;
    GifPixelType *src_buff = NULL;
    ColorMapObject *cmap;
    guchar *dst_buff = NULL, *dst;
    gint err = FALSE, ext_code;
    gint i, j, width = 0, height, transp, has_transp = FALSE;

    lseek( fileno( image->preview->file), 0, SEEK_SET);
    gif = DGifOpenFileHandle( fileno( image->preview->file));

    if ( !gif)
	return FALSE;

    /* find if image has transparency */
    do {
	if ( DGifGetRecordType( gif, &rec) == GIF_ERROR) {
	    err = TRUE;
	    break;
	}
	switch( rec) {
	case SCREEN_DESC_RECORD_TYPE:
	    /* skip */
	    break;
	case IMAGE_DESC_RECORD_TYPE:
	    if ( DGifGetImageDesc(gif) == GIF_ERROR) {
		err = TRUE;
		break;
	    }
		
	    width = gif->Image.Width;
	    height = gif->Image.Height;

	    image_set_infos( image,
			     GIF_IMAGE,
			     width, height,
			     TRUE,
			     has_transp);
	    src_buff = g_malloc( width);
	    dst_buff = g_malloc( width * (has_transp ? 4 : 3));

	    /* xxx interlaced gifs are not supported at the moment */
	    if ( gif->Image.Interlace || !src_buff || !dst_buff) {
		err = TRUE;
		if ( gif->Image.Interlace)
		    g_warning( "gif: interlaced gifs are not supported");
		break;
	    }
	    cmap = gif->Image.ColorMap ? gif->Image.ColorMap : gif->SColorMap;

	    image_render_init( image, width, height, 
			       has_transp ? IMAGE_COLOR_24_ALPHA : IMAGE_COLOR_24);

	    /* xxx what should i do with gif->SBackGroundColor ? */
	    for ( i = 0; i < height; i++) {
		if ( DGifGetLine( gif, src_buff, width) == GIF_ERROR) {
		    g_warning( "gif: Premature end of gif file");
		    break;
		}
		
		dst = dst_buff;
		if ( has_transp) 
		    for ( j = 0; j < width; j++) {
			if ( src_buff[j] == transp) {
			    dst[0] = dst[1] = dst[2] = 0;
			    dst[3] = 0xff;

			} else {
			    
			    dst[0] = cmap->Colors[ src_buff[j]].Red;
			    dst[1] = cmap->Colors[ src_buff[j]].Green;
			    dst[2] = cmap->Colors[ src_buff[j]].Blue;
			    dst[3] = 0;
			}
			dst += 4;
		    }
		else 
		    for ( j = 0; j < width; j++) {
			dst[0] = cmap->Colors[ src_buff[j]].Red;
			dst[1] = cmap->Colors[ src_buff[j]].Green;
			dst[2] = cmap->Colors[ src_buff[j]].Blue;
			dst += 3;
		    }

		if ( !image_append_rows( image, 1, dst_buff)) {
		    err = TRUE;
		    break;
		}
	    }
		
	    rec = TERMINATE_RECORD_TYPE;
	    break;
	case EXTENSION_RECORD_TYPE:
	    if ( DGifGetExtension(gif, &ext_code, &ext) == GIF_ERROR) {
		err = TRUE;
		break;
	    }
	    do {
		if (  ext && (ext[1] & 1)) {
		    has_transp = TRUE;
		    transp = ext[4];
		    break;
		}
	    } while (  DGifGetExtensionNext( gif, &ext) == GIF_OK && ext);
	    break;
	case TERMINATE_RECORD_TYPE:
	default:
	    break;
	}

    } while ( !err && rec != TERMINATE_RECORD_TYPE);

    DGifCloseFile( gif);

    if ( src_buff)
	g_free( src_buff);
    if ( dst_buff)
	g_free( dst_buff);

    return err == FALSE;
}

#endif /* HAVE_LIBGIF */
