/*  jview : Image viewer
 *  Copyright (C) 1998 david Ramboz
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __IMDB_H__
#define __IMDB_H__

#include "image.h"
#include "ldb.h"

#define IMDB_VERSION 1
#define IMDB_MARKER_SIZE 4
#define IMDB_MARKER_VALUE "ImDB"

typedef struct _ImDBHeader ImDBHeader;
typedef struct _ImDB ImDB;
typedef struct _ImDBProperty ImDBProperty;
typedef struct _ImDBPropertyValue ImDBPropertyValue;
typedef struct _ImDBPropertyEntry ImDBPropertyEntry;
typedef struct _ImDBImage ImDBImage;
typedef struct _ImDBDirectory ImDBDirectory;

typedef enum {
    ImDBImageType,
    ImDBDirectoryType,
    ImDBPropertyType,
    ImDBTypeCount
} ImDBTypes;

typedef enum {
    ImDBItemNew        = 1 << 2,
    ImDBItemModified   = 1 << 3,
    ImDBItemDeleted    = 1 << 4
} ImDBItemFlags;

#define IMDB_HEADER_SIZE 10 + IMDB_MARKER_SIZE

struct _ImDBHeader {
    guint32   image_count;
    guint16   directory_count;
    guint16   property_count;
    guint16   imdb_version;
    char      imdb_marker[IMDB_MARKER_SIZE];
};

struct _ImDB {
    ImDBHeader header;

    char    *db_path;

    LDB     *ldb;
    GArray  *dirtree;
    GList   *properties;

};


struct _ImDBProperty {
    guint8 property_id;
    guint8 next_value_id;
    char *name;
    GList *values;

    LDBItemId ldb_id;

    ImDBItemFlags flags;
};

struct _ImDBPropertyValue {
    guint8 id;
    char *name;
};

struct _ImDBPropertyEntry {
    guint8 id;
    guint8 value;
};


struct _ImDBImage {
    Image image;
    
    guint16 directory;

    GList *properties;
    GList *keywords;

    LDBItemId ldb_id;

    ImDBItemFlags flags;
};


struct _ImDBDirectory {
    guint16 id, parent, brother, child;
    char *path;
    char *name;

    LDBItemId ldb_id;

    ImDBItemFlags flags;

    GList *images;
};


ImDB*                   imdb_load( char *path);
ImDB*                   imdb_create( char *path);
void                    imdb_close( ImDB *imdb);
void                    imdb_dump( ImDB *imdb, char *dest_file);
void                    imdb_flush( ImDB *imdb);

void                    imdb_append_images( ImDB *imdb, char *path, GList *images);
void                    imdb_remove_images( ImDB *imdb, GList *images);
gint                    imdb_image_add_property( ImDB *imdb,
						 ImDBImage *image,
						 gint property_id,
						 gint value_id);
gint                    imdb_image_delete_property( ImDB *imdb,
						    ImDBImage *image,
						    gint property_id,
						    gint value_id);
ImDBPropertyEntry*      imdb_image_find_property( ImDB *imdb,
						  ImDBImage *image,
						  gint property_id,
						  gint value_id);
ImDBProperty*           imdb_property_add( ImDB *imdb, gchar *name);
ImDBPropertyValue*      imdb_property_add_value( ImDB *imdb, 
						 ImDBProperty *property, 
						 gchar *name);
void                    imdb_property_delete( ImDB *imdb, ImDBProperty *property);
void                    imdb_property_delete_value( ImDB *imdb, 
						    ImDBProperty *property, 
						    ImDBPropertyValue *value);
guint                   imdb_append_path( ImDB *imdb, char *path);
GList*                  imdb_find_images_from_directory( ImDB *imdb, guint directory);


#endif /*  __IMDB_H__ */
