/*  jview : Image viewer
 *  Copyright (C) 1998 david Ramboz
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>
#include <gtk/gtk.h>

#include "gdkrgb/gdkrgb.h"


#include "browsers.h"
#include "image.h"
#include "xpm.h"
#include "jpeg.h"
#include "pngsup.h"
#include "prefs.h"
#include "main.h"

static void                 register_formats();


static void
register_formats() {
    register_xpm();
#ifdef HAVE_LIBJPEG
    register_jpeg();
#endif
#ifdef HAVE_LIBPNG
    register_png();
#endif
#ifdef HAVE_LIBGIF
    register_gif();
#endif
}


int
main( int argc, char **argv) {
    char *home, *config_file = NULL;
    
    GtkWidget *window;
    
    gtk_set_locale();
    gtk_init( &argc, &argv);
     
    gdk_rgb_init();

    gtk_widget_set_default_colormap( gdk_rgb_get_cmap());
    gtk_widget_set_default_visual( gdk_rgb_get_visual());
    

    /* load preferences */
    home = getenv( "HOME");

    if ( home) {
	config_file = g_malloc( strlen( home) + CONFIG_FILE_NAME_SIZE + 3);
	sprintf( config_file, "%s/.%s", home, CONFIG_FILE_NAME);
    }

    if ( !home || !pref_load( config_file)) 
	g_warning( "Can't load preferences");
    
    if ( config_file) g_free( config_file);

    if ( !pref_parse_args( &argc, &argv)) {
	pref_display_help( stderr);
	gtk_exit( 1);
    }

    register_formats();

    window = directory_browser_new( getcwd( NULL, 0), 120, 120, 50);
    gtk_widget_show( window);

    gtk_main();

    return 0;
}


