/*  jview : Image viewer
 *  Copyright (C) 1998 david Ramboz
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __PREFS_H__
#define __PREFS_H__

#include <stdio.h>
#include <glib.h>
#include <gtk/gtktypeutils.h>

typedef struct _Pref Pref;
typedef struct _PrefDef PrefDef;


/* Pref Flags */
enum {
    PREF_NONE        = 1 << 1,
    PREF_PUBLIC      = 1 << 2,
    PREF_OVERWRITE   = 1 << 3,
    PREF_VALUE_SET   = 1 << 7,
    PREF_INVALID     = 1 << 8
};

struct _PrefDef {
    gint flags;

    char *name;
    char *description;

    char *opt;
    char *long_opt;

    gint  value_type;
    char *default_value;
};

struct _Pref {
    gint flags;

    char *description;

    char *opt;
    char *long_opt;

    GtkArg value;
};

	     
/* Create a preference specific to 'id'. Mutiple preferences can be
   associated to the same id. 
   You can for example set 'id' to GLOBAL for all global variables or
   set 'id' to <MODULE_NAME> for all preferences specific 
   to the module <MODULE>.
   
  */
gint                pref_new( char *name, 
			      gint flags,
			      char *description, 
			      char *opt,
			      char *long_opt,
			      char *id, 
			      gint type, 
			      gpointer default_value);
			
gint                pref_new_from_defs( PrefDef *prefs, gint count, char *id);
void                pref_free_all();

gint                pref_parse_args( gint *argc, char ***argv);
void                pref_display_help( FILE *file);

GtkArg*             pref_get_value( char *name, char *id);
gint                pref_set_value( char *name, char *id, gpointer value );

void                pref_dump( FILE *file, char *id);
gint                pref_save( char *file_name, char *id);
gint                pref_load( char *file_name); 



#endif /* __PREFS_H__ */
